from typing import Optional

from pydantic import Field

from dar_etl.schemas.base_model import DarBaseModel


class NamedRoadMunicipalDistrict(DarBaseModel):
    municipality: Optional[str] = Field(
        None,
        alias="kommune",
        description="EAID_0A04B5E9_9F12_49cb_B2AF_AFEE0315AE2B",
    )
    waycode: Optional[str] = Field(
        None,
        alias="vejkode",
        description="EAID_852A683F_3E30_4009_A003_B3C8F6F76930",
    )
    named_road: Optional[str] = Field(
        None,
        alias="navngivenVej",
        description="EAID_dstE87A3E_ECA3_4f38_A586_117287E4754A",
    )



from typing import Optional

from pydantic import Field

from dar_etl.schemas.base_model import DarBaseModel


class NamedRoadPostalCode(DarBaseModel):
    named_road: Optional[str] = Field(
        None,
        alias="navngivenVej",
        description="EAID_1B3CAEB0_80FA_486f_ABD1_818B332C905C",
    )
    postal_code: Optional[str] = Field(
        None,
        alias="postnummer",
        description="EAID_AD2844E3_7F05_4d8d_967E_44BA3D12C4B4",
    )



from collections.abc import Iterable
from typing import Type

from dar_etl.parsing.parser import DarParser
from dar_etl.schemas.address import Adresse
from dar_etl.schemas.address_point import AddressPoint
from dar_etl.schemas.base_model import DarBaseModel
from dar_etl.schemas.house_number import HouseNumber
from dar_etl.schemas.named_road import NamedRoad
from dar_etl.schemas.named_road_municipal_district import NamedRoadMunicipalDistrict
from dar_etl.schemas.named_road_postal_code import NamedRoadPostalCode
from dar_etl.schemas.named_road_supplementary_city_name import NamedRoadSupplementaryCityName
from dar_etl.schemas.postal_code import PostalCode
from dar_etl.schemas.root_keys import Root
from dar_etl.schemas.supplementary_city_name import SupplementaryCityName


class ParserFactory:
    _registry: dict[Root, Type[DarBaseModel]] = {
        Root.addresses: Adresse,
        Root.address_points: AddressPoint,
        Root.house_numbers: HouseNumber,
        Root.named_roads: NamedRoad,
        Root.named_road_municipal_parts: NamedRoadMunicipalDistrict,
        Root.named_road_postal_codes: NamedRoadPostalCode,
        Root.named_road_supplementary_city_names: NamedRoadSupplementaryCityName,
        Root.postal_codes: PostalCode,
        Root.supplementary_city_names: SupplementaryCityName,
    }

    @classmethod
    def create(cls, root: Root) -> DarParser:
        return DarParser(
            parsing_type=cls._registry[root],
            root=root,
        )

    @classmethod
    def create_all(cls) -> Iterable[DarParser]:
        yield from (
            DarParser(
                parsing_type=parsing_type,
                root=root,
            )
            for root, parsing_type in cls._registry.items()
        )



from io import BufferedReader
import logging
from pathlib import Path
from typing import Iterable, Type

import ijson
from pydantic import ValidationError

from dar_etl.schemas.base_model import DarBaseModel
from dar_etl.schemas.root_keys import Root


class DarParser:
    def __init__(self, root: Root, parsing_type: Type[DarBaseModel]) -> None:
        self.root_key: str = root.value
        self.parsing_type = parsing_type

    def parse(self, filepath: Path) -> Iterable[DarBaseModel]:
        with open(file=filepath, mode="rb") as file_pointer:
            try:
                yield from self.iterate_json(file_pointer=file_pointer)
            except ijson.IncompleteJSONError:
                logging.error(self.root_key)

    def iterate_json(self, file_pointer: BufferedReader) -> Iterable[DarBaseModel]:
        for record in ijson.items(file_pointer, f"{self.root_key}.item"):
            try:
                yield self.parsing_type(**record)
            except ValidationError as validation_error:
                logging.error(record)
                raise validation_error



from typing import Optional

from pydantic import Field

from dar_etl.schemas.base_model import DarBaseModel


class Adresse(DarBaseModel):
    door_designation: Optional[str] = Field(
        None,
        alias="dørbetegnelse",
        description="EAID_16B39B2F_0D4D_45ca_8164_8E1D78D202FC",
    )
    door_point: Optional[str] = Field(
        None,
        alias="dørpunkt",
        description="EAID_2E77B0C3_49AF_47b0_AD4E_DD653B99FD0B",
    )
    floor_designation: Optional[str] = Field(
        None,
        alias="etagebetegnelse",
        description="EAID_805BB835_641D_49d5_93E6_80C8E3FFD91C",
    )

    address_designation: Optional[str] = Field(
        None,
        alias="adressebetegnelse",
        description="EAID_BB398854_F92E_4372_A20A_D7136BB5CD9C",
    )
    building: Optional[str] = Field(
        None,
        alias="bygning",
        description="BBR bygning er relateret til husnummer og ikke til adressen, der er derfor ikke bagvedliggende data for denne parameter, og anvendelsen af parameteren vil ikke give noget resultat.",
    )
    house_number: Optional[str] = Field(
        None,
        alias="husnummer",
        description="Angiver det DAR husnummer (adgangsadresse), som adressen er tilknyttet. Bruger du dette inputparameteren finder du alle adresser for det pågældende husnummer",
    )



from typing import Optional

from pydantic import Field

from dar_etl.schemas.base_model import DarBaseModel


class PostalCode(DarBaseModel):
    name: Optional[str] = Field(
        None,
        alias="navn",
        description="EAID_9E7FA46D_52D9_4615_B24C_599BB28D1923",
    )
    postal_code: Optional[str] = Field(
        None,
        alias="postnr",
        description="EAID_8F8BACBF_E985_4dc1_BCF4_27F5C3FD0BB1",
    )
    postal_code_division: Optional[str] = Field(
        None,
        alias="postnummerinddeling",
        description="EAID_E5F5DDF9_3CF4_4304_B02D_E67AB0CDD619",
    )



from typing import Optional

from pydantic import Field

from dar_etl.schemas.base_model import DarBaseModel


class NamedRoadSupplementaryCityName(DarBaseModel):
    named_road: Optional[str] = Field(
        None,
        alias="navngivenVej",
        description="EAID_7CF789EC_31C0_4af3_B75B_77674A26DECD",
    )
    supplementary_city_name: Optional[str] = Field(
        None,
        alias="supplerendeBynavn",
        description="EAID_21C0B8C7_8B67_406c_BDC1_21ADBA284867",
    )



from datetime import datetime
from enum import Enum
from typing import Optional

from pydantic import BaseModel
from pydantic import Field


class Status(Enum):
    """https://danmarksadresser.dk/adressedata/kodelister/livscyklus/"""

    internal_preperation = "1"
    preliminary = "2"
    applicable = "3"
    discontinued = "4"
    archived = "5"
    deleted = "6"
    not_in_use = "7"
    in_use = "8"
    obsolete = "9"
    expired = "10"
    terminated = "11"


class BusinessProcess(Enum):
    """https://danmarksadresser.dk/adressedata/kodelister/forretningsproces"""

    master_data = "0"  # Stamdata
    address_assignment = "1"  # Adresseopgave
    dialog_client = "2"  # Dialogklient
    street_postcode = "3"  # Gadepostnummer
    supplementary_city_name = "4"  # Supplerendebynavn
    error_reporting_client = "5"  # Fejlmeldeklient
    undone_forward_dating = "6"  # Fortrudt fremdatering


class BusinessArea(Enum):
    dar = "54.15.10.25"  # Danmarks Adresse Register, DAR
    postal_codes = "54.15.10.15"  # Postnumre
    road_names_and_road_codes = "54.15.10.06"  # Vejnavne og vejkoder
    house_number = "54.15.10.07"  # Husnummer
    addresses = "54.15.10.08"  # Adresser
    unknown = "unknown"
    unknown1 = "10.00.00.00"
    unknown2 = "00.00.00.00"

    @classmethod
    def _missing_(cls, _: object) -> Enum:  # noqa: WP526 WPS120
        return cls(cls.unknown)


class BusinessEvent(Enum):
    named_road = "0"  # Navngivenvej
    named_road_municipal_district = "1"  # NavngivenvejKommunedel
    address = "2"  # Adresse
    house_number = "3"  # Husnummer
    postal_code = "4"  # Postnummer
    supplementary_city_name = "5"  # Supplerendebynavn
    house_number_reservation = "6"  # Husnummerreservation
    road_name_reservation = "7"  # VejnavnReservation
    address_assignment = "8"  # Adresseopgave
    address_assignment_item = "9"  # Adresseopgavegenstand
    dagi_municipality = "10"  # DAGIKommune
    dagi_parish_division = "11"  # DAGISogneinddeling
    dagi_postal_code = "12"  # DAGIPostnummer
    dagi_voting_area = "13"  # DAGIAfstemningsområde
    dagi_parish_council_voting_area = "14"  # DAGIMeninghedsrådsafstemningsområde
    dagi_supplementary_city_name = "15"  # DAGISupplerendeBynavn
    bbr_building = "16"  # BBRBygning
    bbr_technical_plant = "17"  # BBRTekniskanlæg
    mu_piece_of_land = "18"  # MUJordstykke (matriklens udvidelse)
    geo_danmark_building = "19"  # GeoDanmarkBygning
    geo_danmark_waypoint = "20"  # GeoDanmarkVejmidte


class DarBaseModel(BaseModel):
    business_event: Optional[BusinessEvent] = Field(
        None,
        alias="forretningshændelse",
        description="Den forretningshændelse, som afstedkom opdateringen af adresseelementet til den pågældende version",
    )
    business_area: Optional[BusinessArea] = Field(
        None,
        alias="forretningsområde",
        description="Det forretningsområde som har opdateret adresseelementet til den pågældende version",
    )
    business_process: Optional[BusinessProcess] = Field(
        None,
        alias="forretningsproces",
        description="Den forretningsproces, som afstedkom opdateringen af adresseelementet til den pågældende version",
    )
    id_namespace: Optional[str] = Field(
        None,
        alias="id_namespace",
        description="EAID_D273E3D8_A3F4_41bc_AF9C_E67B4A29C008.EAID_6CEB0356_5CBF_4159_B96B_A2489DD2DAC8",
    )
    id_local: Optional[str] = Field(
        None,
        alias="id_lokalId",
        description="Identifikation af adresseelementet igennem hele dets livscyklus.",
    )
    registration_from: Optional[datetime] = Field(
        None,
        alias="registreringFra",
        description="Tidspunktet hvor registreringen af den pågældende version af adresseelementet er foretaget",
    )
    registration_operator: Optional[str] = Field(
        None,
        description="Den aktør der har foretaget registreringen af den pågældende version af adresseelementet",
        alias="registreringsaktør",
    )
    registration_to: Optional[datetime] = Field(
        None,
        alias="registreringTil",
        description="Tidspunktet hvor en ny registrering på adresseelementet er foretaget, og hvor denne version således ikke længere er den seneste.",
    )
    status: Optional[Status] = Field(
        None,
        alias="status",
        description="Adresseelementets status i den pågældende version, dvs. elementets tilstand i den samlede livscyklus",
    )
    effect_from: Optional[datetime] = Field(
        None,
        alias="virkningFra",
        description="Tidspunktet hvorfra den pågældende version af adresseelementet har virkning",
    )
    effect_to: Optional[datetime] = Field(
        None,
        alias="virkningTil",
        description="Tidspunktet hvor virkningen af den pågældende version af adresseelementet ophører",
    )
    effector: Optional[str] = Field(
        None,
        alias="virkningsaktør",
        description="Den aktør der har afstedkommet virkningsegenskaberne for den pågældende version af adresseelementet",
    )



import logging
from pathlib import Path
from typing import Iterable

from kafka import KafkaConsumer

from dar_etl.config import Config


class NewDarFileNotificationConsumer:
    def __init__(self, config: Config) -> None:
        host = config.kafka.host
        port = config.kafka.port

        self.input_dir = config.app.input_directory
        self.client_id = f"new-dar-file-notification-consumer-{config.version}"
        self.consumer = KafkaConsumer(
            config.kafka.topic,
            bootstrap_servers=f"{host}:{port}",
            client_id=self.client_id,
            group_id="dar-etl",
            enable_auto_commit=True,
            auto_offset_reset="earliest",
        )

    def consume(self) -> Iterable[Path]:
        for message in self.consumer:
            filename_bytes: bytes = message.value
            filename = filename_bytes.decode("utf-8")
            logging.info(f"[CONSUMED] {filename}")
            yield self.input_dir.joinpath(filename)



import logging

from kafka import KafkaProducer

from dar_etl.config import Config
from dar_etl.schemas.base_model import DarBaseModel


class DarEntryPublisher:
    def __init__(self, config: Config) -> None:
        host = config.kafka.host
        port = config.kafka.port
        self.producer = KafkaProducer(bootstrap_servers=f"{host}:{port}")

    def publish(self, dar_entry: DarBaseModel, topic: str) -> None:
        encoded_dar_entry = dar_entry.json().encode("utf-8")
        if dar_entry.id_local:
            key = dar_entry.id_local.encode("utf-8")
        else:
            key = None
        self.producer.send(topic=topic, key=key, value=encoded_dar_entry)
        self.producer.flush()
        logging.debug("[PUBLISHED] Entry")
